package com.gxzc.zen.api.sys.service.impl

import com.baomidou.mybatisplus.mapper.EntityWrapper
import com.baomidou.mybatisplus.service.impl.ServiceImpl
import com.gxzc.zen.api.sys.common.MenuTree
import com.gxzc.zen.api.sys.mapper.SysMenuMapper
import com.gxzc.zen.api.sys.model.SysMenu
import com.gxzc.zen.api.sys.service.ISysMenuService
import com.gxzc.zen.common.contants.ZenConstants
import com.gxzc.zen.common.exception.ZenException
import com.gxzc.zen.common.exception.ZenExceptionEnum
import com.gxzc.zen.common.util.TreeUtil
import com.gxzc.zen.orm.annotation.ZenTransactional
import com.gxzc.zen.umps.constant.ZenHttpSession
import com.gxzc.zen.umps.util.SSOUtil
import org.springframework.beans.BeanUtils
import org.springframework.stereotype.Service

/**
 * <p>
 * 菜单表 服务实现类
 * </p>
 *
 * @author NorthLan123
 * @since 2018-02-06
 */
@Service
class SysMenuServiceImpl : ServiceImpl<SysMenuMapper, SysMenu>(), ISysMenuService {

    private fun getTree(data: List<SysMenu>): MutableList<MenuTree> {
        val nodes = mutableListOf<MenuTree>()
        data.forEach {
            val node = MenuTree()
            BeanUtils.copyProperties(it, node)
            nodes.add(node)
        }
        return TreeUtil.build(nodes, ZenConstants.TREE_ROOT_PID)
    }

    override fun getRootTree(): MenuTree? {
        val ret = getTree(baseMapper.selectList(null))
        return if (ret.isEmpty()) null else ret[0]
    }

    private fun getUserMenuTree(resourceMenuIds: List<Long>?, roleIds: List<Long>?): MutableList<MenuTree> {
        // 获取所有权限菜单
        val distinctMenu = hashSetOf<SysMenu>().apply {
            if (resourceMenuIds != null && resourceMenuIds.isNotEmpty()) {
                addAll(baseMapper.selectBatchIds(resourceMenuIds))
            }
            if (roleIds != null && roleIds.isNotEmpty()) {
                addAll(baseMapper.selectByRoleIds(roleIds))
            }
        }
        // 获取权限菜单的所有父节点以便于组合成树
        val parentsIds = mutableSetOf<String>()
        distinctMenu.forEach {
            if (!it.path.isNullOrEmpty()) {
                val paths = it.path!!.split(ZenConstants.TREE_PATH_SEPARATOR)
                parentsIds.addAll(paths.subList(0, paths.size - 1)) // 不添加自身
            }
        }
        if (parentsIds.isNotEmpty()) {
            distinctMenu.addAll(baseMapper.selectBatchIds(parentsIds))
        }
        return getTree(distinctMenu.toList())
    }

    override fun getUserMenuTree(platformId: Long): MutableList<MenuTree> {
        var menu = SSOUtil.getCurUserMenuTree(platformId)
        if (menu == null || menu.isEmpty()) {
            val res = SSOUtil.getCurUserPerms()?.map { it.menuId!! }
            val roleIds = SSOUtil.getCurUserRoles()?.map { it.id!! }
            menu = getUserMenuTree(res, roleIds)
            if (menu.isNotEmpty()) {
                // TODO 刷新缓存
                SSOUtil.setAttribute(ZenHttpSession.SESSION_KEY_USER_MENU + platformId, menu)
            } else {
                return mutableListOf()
            }
        }
        // 根据平台id取出对应平台的菜单列表
        // 此处使用广度优先遍历会快一些 因为主要过滤都在第二层树
        menu = TreeUtil.findBFS(menu) {
            it.ext1 != null && it.ext1 == platformId.toString()
        }
        if (menu.isNotEmpty()) {
            val root = menu[0]
            if (root.children != null && root.children!!.isNotEmpty()) {
                menu = mutableListOf()
                root.children?.forEach {
                    menu.add(it as MenuTree)
                }
            }
        }
        return menu
    }

    override fun getSysMenuList(): MutableList<SysMenu> {
        return baseMapper.selectList(null)
    }

    override fun getSysMenuTree(): MutableList<MenuTree> {
        return getTree(getSysMenuList().toList())
    }

    @ZenTransactional
    override fun createSysMenu(entity: SysMenu): SysMenu {
        // 先新增一条数据 再根据此数据更新对应的path
        if (baseMapper.insert(entity) <= 0) {
            throw ZenException(ZenExceptionEnum.BIZ_INSERT_ERROR)
        }
        if (baseMapper.updatePath(entity.parentId!!, entity.id!!) <= 0) {
            throw ZenException(ZenExceptionEnum.BIZ_INSERT_ERROR)
        }
        return entity
    }

    override fun updateMenu(entity: SysMenu): SysMenu {
        if (baseMapper.updateById(entity) <= 0) {
            throw ZenException(ZenExceptionEnum.BIZ_UPDATE_ERROR)
        }
        return entity
    }

    @ZenTransactional
    override fun deleteMenu(entity: SysMenu) {
        if (entity.id == null) {
            throw ZenException(ZenExceptionEnum.BIZ_DELETE_ERROR)
        }
        // 先查询现有的数据 (path)
        val stored = baseMapper.selectById(entity.id) ?: throw ZenException(ZenExceptionEnum.BIZ_DELETE_ERROR)
//        val endPath = stored.path?.substring(0, stored.path!!.length - 1) + (stored.id!! + 1)
        if (baseMapper.physicalDelete(EntityWrapper<SysMenu>().like("path", "${stored.path!!}%")) <= 0) {
            throw ZenException(ZenExceptionEnum.BIZ_DELETE_ERROR)
        }
    }
}
